function out = PPgen_selfexc(t,delta,param)
% Generate burst from the a self excitatory neuron using GLM model.
% Principle of this process is based on equation (7) of D. H. Johnson's
% paper "Point Process Models of Signle-Neuron Discharges"
%
% t - total time of simulation
% delta - bin width in second
% param - first one is always the constant term

len_history = length(param)-1;
% if history is not specified, assume all past history bins are zero
history = zeros(1,len_history);

len_out = floor(t/delta);

% prepend history to our output data. REMOVE history afterwards please!!
out = [history zeros(1,len_out)];

% start the integration and go through every time bin, which will also
% determine the precision of the integration

E = exprnd(1,1);
accum = 0;
for cnt = 1:len_out
    
    % do the integration
    accum = accum+delta*exp([1 out(cnt+len_history-1:-1:cnt)]*param');
    
    % the interval hits the input value E, a spike is generated, exp random
    % variable is reset, and integral is set anew.
    if (accum>E) 
        out(cnt+len_history)=1;
        E = exprnd(1,1);
        accum = 0;
    end
    
end

out = out(len_history+1:end);
% stem(out)

return