function [C,results,input,rest]=connection(filenm,DI,stimorder,order,rand,makefig)
% Measures the connection of the simulated CPG
% filenm: location to read the PSP simulation output data (connection.smu.out) 
% Eg. connection('C:\Users\cneveu\Desktop\Aplysia_Feeding_CPG_Network\connection.smu.out',stimorder,order,'no','yes')
% stimorder={'34';'8';'35';'51';'4';'63';'Z';'64';'52'}
% order={'4';'8';'52';'Z';'35';'64';'34';'51';'63'}; this is an example, the 
% real order is saved in the matlab file located in the folder 
% rand=test how it compares to randomly generated matrix.  recommend 'no'
%      for this input
% makefig= make a fig or not

% input matrix by DI
%   1  2  3  4  5  6  7  8  9
% DI=[0  0  0  0  0  0  0  0  0; ...
%     0  0  0  -1 0  0  -1 1  1; ...
%     0  0  0  0  0  0  1  0  -1; ...
%     0  0  0  0  1  0  0  0  0; ...
%     0  0  0  1  0  0  0  0  0; ...
%     0  0  0  0  0  0  0  0  0; ...
%     0  0  1  0  0  0  0  0  0; ...
%     1  0  0  0  0  0  0  0  0; ...
%     -1 0  -1 0  0  0  0  1  0];
data=dlmread(filenm);
DI(DI>0)=1;
DI(DI<0)=-1;

interval=10;% interval of spike triggering stimulus
window=0.03;% window to measure PSP in seconds
cres=32;% resolution of coloring
thr=10;%threshold for postive synaptic response

freq=1/(data(2,1)-data(1,1));

data=data(:,2:end);

cellfind = @(string)(@(cell_contents)(strcmp(string,cell_contents)));
for a=1:length(order)
    norder(a)=find(cellfun(cellfind(order{a}),stimorder));
end

stimt=freq*interval;
C=NaN(size(data,2),size(data,2));
if strcmpi(makefig,'yes');
    figure
end

colors=[0 0 0; 1 0 0; 0 1 0; 0 0 1; 1 .5 0; 0 1 1;1 0 1;.5 .5 0; 0 .5 .5];
for neur=norder
   sub=data(neur*stimt:neur*stimt+window*freq,:)-repmat(data(neur*stimt,:),[window*freq+1 1]);
   C(neur,:)=sum(sub); 
   sub(:,neur)=zeros(length(sub),1);
   if strcmpi(makefig,'yes');
       subplot(2,5,neur)
        for a=norder  
            plot(sub(:,a),'Color',colors(a,:));hold on; 
        end
        set(gca,'XLim',[0 length(sub)])
        title(stimorder{neur})
        if neur==length(C); legend(order); end
   end
end

C(logical(eye(length(C))))=0;
C(abs(C)<thr)=0;

map=[zeros(cres,1) (1/cres:1/cres:1)' ones(cres,1); ...
     ones(1,3); ...
     ones(cres,1) (1:-1/cres:1/cres)' zeros(cres,1)];
 
C=C(norder,norder);
if strcmpi(makefig,'yes');
    figure
    image(C,'CDataMapping','scaled')
    caxis([-150 150])% [-6 6] for mV/ms
    for a=1:8
       line([0 length(C)+0.5],[a a]+0.5,'Color','k') 
       line([a a]+0.5,[0 length(C)+0.5],'Color','k') 
    end

    colormap(map);
    colorbar('FontSize',16)
    ax=gca;
     ax.YTickLabel=order;
     ax.XTickLabel=ax.YTickLabel;
     ax.FontSize=16;
     ax.XTick=1:9;
     ax.YTick=1:9;
end
  
results=cell(5);
results{1,4}='Actual';
results{4,1}='Predicted';
results(2,3:5)={'Excitatory','Inhibitory','Neither'};

DIr(:,1)=DI(:)>0;  DIr(:,2)=DI(:)<0;  DIr(:,3)=abs(DI(:))<1;
Ar(:,1)=C(:)>thr;    Ar(:,2)=C(:)<-thr;    Ar(:,3)=abs(C(:))<thr;
results(3:5,2)=results(2,3:5)';
for a=1:3
    for di=1:3
        results{di+2,a+2}=sum(Ar(:,a) & DIr(:,di)); 
    end
end

count=sum(DIr);

rand_n=1e7;% number of random connectivity matrices

if nargin>4 && strcmpi(rand,'on')
    Rpe=randi(length(C)^2,sum(DIr(:,1)),rand_n);
    Rpi=randi(length(C)^2,sum(DIr(:,2)),rand_n);

    r_results=zeros(3,3,rand_n);
    clearvars Rr
    for r=1:rand_n
        if mod(r,1e6)==0;display([num2str(r/1e6) 'million']);end
       R=zeros(length(C)^2,1);
       R(Rpi(:,r))=-1;  R(Rpe(:,r))=1;
       Rr(:,1)=R(:)>thr;  Rr(:,2)=R(:)<-thr;  Rr(:,3)=abs(R(:))<thr;
       for a=1:3
            for di=1:3
                r_results(di,a,r)=sum(Ar(:,a) & Rr(:,di)); 
            end
       end
    end
else
    rand_n=0;
end

rest=data(2000,:);
rest(2,:)=data(9000,:);
rest(3,:)=diff(rest(1:2,:));

input=table(interval,window,cres,thr,freq,rand_n);


